@@ -2,6 +2,9 @@ from numpy import complex, cos, sin, exp, array, pi, angle, matmul
 from numpy.linalg import eig, det
 from .utils import *
 from .jonesvector import JonesVector
+import matplotlib
+import matplotlib.pyplot as plt
+import matplotlib.animation as animation
 
 class JonesMatrix:
     """ A Jones matrix represents any element that can transform polarization. 
@@ -17,15 +20,40 @@ class JonesMatrix:
         It is very often implicitly x and y, but we will track it explicitly
         to avoid problems
         """
-
         if m is not None:
-            self.m = array(m)
+            self.mOriginal = array(m)
         else:
-            self.m = array([[A,B],[C,D]])
+            self.mOriginal = array([[A,B],[C,D]])
+
+        self.orientation = 0
         self.L = physicalLength
 
-        self.b1 = array([1,0]) # x̂
-        self.b2 = array([0,1]) # ŷ
+    @property
+    def m(self):
+        theta = self.orientation
+        rotMatrix = array([[cos(theta),sin(theta)],[-sin(theta), cos(theta)]],dtype='complex')
+        invRotMatrix = array([[cos(theta),-sin(theta)],[sin(theta), cos(theta)]],dtype='complex')
+
+        return matmul(invRotMatrix, matmul(self.mOriginal, rotMatrix))
+
+    @property
+    def b1(self):
+        """ The basis vector for x. For now this is not modifiable. """
+        return array([1,0]) # x̂
+    
+    @property
+    def b2(self):
+        """ The basis vector for y. For now this is not modifiable. """
+        return array([0,1]) # ŷ
+
+    def setValue(self, name, value):
+        try:
+            setattr(self, name, value)
+        except:
+            print("Some properties are not mutable")
+
+    def value(self, name):
+        return getattr(self, name)
 
     @property
     def A(self):
@@ -40,13 +68,6 @@ class JonesMatrix:
     def D(self):
         return self.m[1,1]
 
-    @property
-    def asArray(self):
-        return self.m
-
-    def setFromArray(self, anArray):
-        self.m = anArray
-
     @property
     def determinant(self):
         return det(self.m)
@@ -71,6 +92,7 @@ class JonesMatrix:
         associated eigenvalues are on the diagonal.
 
         TODO: Need to prove that the basis vectors will always be real.
+        TODO: check e1 and e2 and get them to match x and y after rotation
 
         """
         w, v = eig(self.m)
@@ -86,7 +108,6 @@ class JonesMatrix:
         if isAlmostZero(e1[1]):
             e1[1] = 0
 
-
         e2 = [0,1]
         if isEssentiallyReal(v[1][0]):
             e2[0] = v[1][0].real
@@ -108,7 +129,7 @@ class JonesMatrix:
         return complex(0, 0)
 
     @property
-    def retardance(self) -> complex:
+    def retardance(self) -> float:
         phi, e1, e2 = self.birefringence
         return phi
     
@@ -154,10 +175,7 @@ class JonesMatrix:
 
         """
 
-        product = JonesMatrix()
-        product.m = matmul(self.m, rightSideMatrix.m)
-        product.L = self.L + rightSideMatrix.L
-
+        product = JonesMatrix(m=matmul(self.m, rightSideMatrix.m), physicalLength=self.L + rightSideMatrix.L)
         return product
 
     def mul_vector(self, rightSideVector):
@@ -183,9 +201,9 @@ class JonesMatrix:
 
         return outputVector
 
-    def rotateElementBy(self, theta):
-        """ We rotate the optical element of the matrix by theta. For instance, 
-        a theta rotation of a horizontal polarizer will be a polarizer
+    def rotatedBy(self, theta):
+        """ We return a rotated copy of the optical element of the matrix by theta. 
+        For instance, a theta rotation of a horizontal polarizer will be a polarizer
         aligned at theta. """
 
         return Rotation(theta = -theta)*self*Rotation(theta = theta)
@@ -198,6 +216,68 @@ class JonesMatrix:
         a basis x,y that has been rotated to +45 and +135."""
         raise NotImplemented()
 
+    def show(self, input, xObj, xProperty, xRange, yObj, yProperty):
+        if xObj is None:
+            xObj = self
+
+        x = []
+        y = []
+
+        for value in xRange:
+            if xObj is None:
+                self.setValue(xProperty, value)
+            else:
+                xObj.setValue(xProperty, value)
+
+            vOut = self*input
+
+            x.append( xObj.value(xProperty))
+            if yObj is None:
+                y.append( vOut.value(yProperty))
+            else:
+                y.append( yObj.value(yProperty))
+        
+        plt.title("{0} versus {1}".format(yProperty, yProperty))
+        plt.xlabel("{0}".format(xProperty))
+        plt.ylabel("{0}".format(yProperty))
+        plt.plot(x,y,'ko')
+        plt.show()
+
+
+    def showOrientationPlot(self, input:JonesVector):
+        x = []
+        y = []
+        for theta in range(0,190,10):
+            theMatrix = JonesMatrix(m=self.m)
+            theMatrix.orientation = theta*radPerDeg
+
+            vOut = theMatrix*input
+            x.append(theta)
+            y.append(vOut.intensity)
+
+        plt.title("Intensity versus orientation of element")
+        plt.xlabel(r"Rotation $\theta$ of element [°]")
+        plt.ylabel(r"Intensity [arb. unit]")
+        plt.plot(x,y,'ko')
+        plt.show()
+
+    def showPolarizationPlot(self):
+        x = []
+        y = []
+        for theta in range(0,190,10):
+            vIn = JonesVector(Ex=cos(theta*radPerDeg), Ey=sin(theta*radPerDeg))
+            vOut = self*vIn
+            x.append(theta)
+            y.append(vOut.intensity)
+
+        plt.title("Intensity versus input polarization orientation (0° is horizontal)")
+        plt.xlabel(r"Input polarization orientation [°]")
+        plt.xlim(0,180)
+        plt.ylabel(r"Intensity [arb. unit]")
+        plt.plot(x,y,'ko')
+        plt.show()
+
+
 class HorizontalPolarizer(JonesMatrix):
     def __init__(self):
         JonesMatrix.__init__(self, A=1,B=0,C=0,D=0,physicalLength=0)        
@@ -233,30 +313,60 @@ class Rotation(JonesMatrix):
         JonesMatrix.__init__(self, A=cos(theta), B=sin(theta), C=-sin(theta), D=cos(theta), physicalLength=0)
 
 class PhaseRetarder(JonesMatrix):
-    def __init__(self, delta=None, phiX=None, phiY=None, physicalLength=0):
+    def __init__(self, delta=None, phiX=None, phiY=None, theta=0, physicalLength=0):
         if delta is not None:
             JonesMatrix.__init__(self, A=exp(1j * delta), B=0, C=0, D=1, physicalLength=0)
         else:
             JonesMatrix.__init__(self, A=exp(1j * phiX), B=0, C=0, D=exp(1j * phiY), physicalLength=0)
 
-class QWP(JonesMatrix):
+    
+class QWP(PhaseRetarder):
     def __init__(self, theta):
         # theta is fast axis with respect to x-axis
-        baseChange = Rotation(theta)
-        retardance = PhaseRetarder(delta=-pi / 2) # Ex is advanced by pi/2, x is fast
-        invBaseChange = Rotation(-theta)
+        PhaseRetarder.__init__(self, delta=-pi / 2) # Ex is advanced by pi/2, x is fast
+        self.orientation = theta
 
-        qwp = invBaseChange*retardance*baseChange
-        JonesMatrix.__init__(self, m=qwp.m, physicalLength=0)        
-
-class HWP(JonesMatrix):
+class HWP(PhaseRetarder):
     def __init__(self, theta):
-        baseChange = Rotation(theta)
-        retardance = PhaseRetarder(delta=-pi)  # Ex is advanced by pi, x is fast
-        invBaseChange = Rotation(-theta)
+        PhaseRetarder.__init__(self, delta=-pi) # Ex is advanced by pi, x is fast
+        self.orientation = theta
 
-        hwp = invBaseChange*retardance * baseChange
-        JonesMatrix.__init__(self, m=hwp.m, physicalLength=0)
+class PockelsCell(JonesMatrix):
+    def __init__(self, halfwaveVoltage, length):
+        self.halfwaveVoltage = halfwaveVoltage
+        self.voltage = 0
+        JonesMatrix.__init__(self, A=1, B=0, C=0, D=1, physicalLength=length)
+
+    @property
+    def m(self):
+        cell = PhaseRetarder(delta=self.voltage/self.halfwaveVoltage*pi)
+        cell.orientation = self.orientation
+        return cell.m
+
+    def showVoltagePlot(self):
+        fig, axs = plt.subplots(1, sharex=True)
+        fig.suptitle("Pockels cell at {0:.1f}° with horizontal incident polarization".format(self.orientation*degPerRad))
+
+        voltages = list(range(0,self.halfwaveVoltage+10,10))
+        yParallel = []
+        yCrossed = []
+        for voltage in voltages:
+            self.voltage = voltage
+
+            vIn = JonesVector.horizontal()
+            vOut = HorizontalPolarizer()*self*vIn
+            yParallel.append(vOut.intensity)
+
+            vOut = VerticalPolarizer()*self*vIn
+            yCrossed.append(vOut.intensity)
+
+
+        # axs[0].xlim(0, self.halfwaveVoltage)
+        # fig.ylabel(r"Intensity [arb. unit]")
+        axs.plot(voltages,yParallel,'k|',label="Between parallel polarizers")
+        axs.plot(voltages,yCrossed,'k+',label="Between crossed polarizers")
+        axs.legend()
+        plt.show()
 
 
 # class Retarder(JonesMatrix):  # fixme: don't know how to call a JonesMatrixFromRetardanceAndDiattenuation
